// Copyright 2002-2015, University of Colorado Boulder

/**
 * Object that manages the steps necessary to rip the website, build the installers, deploy them, and send out email
 * notifications.  It may do all of these steps or only some of them based on the options that are passed in, which
 * originate from the command line.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var _ = require( 'underscore' );
  var InstallerBuilder = require( 'InstallerBuilder' );
  var parseArgs = require( 'minimist' );
  var PhetWebsiteRipper = require( 'PhetWebsiteRipper' );
  var winston = require( 'winston' );

  // static object, no constructor
  return {

    /**
     * Initiate the build using command line arguments.  This is the main entry point to the JavaScript code when
     * doing a build.
     *
     * @param {String} commandLineArgs - see the printUsage method for an explanation of the options.
     * @public
     */
    startBuildFromCommandLine: function( commandLineArgs ) {

      // Define the default options.
      var defaultOptions = {
        deploy: false, // Deploy it to the website?
        deployDev: false, // Deploy it to the dev server?
        email: false, // email notifications about completetion.
        withActivities: 'no', // Include activities in installer or not, 'yes' , 'no' , or 'both', creating two full rips of the website
        locales: [ 'en' ],
        logFile: undefined, // path to logfile to dump.
        silent: false, // No output to console.
        platform: [ 'windows', 'linux', 'osx' ],  // Windows, OSX, or Linux?

        // Just one of each sim type, very fast **generally for testing**
        // quick: true will ignore some other options like:
        //               locales, withActivites, deploy(Dev), email
        // and will instead just rip one of each sim type
        quick: false,


        // options for supporting help
        help: false,  //Help dialog
        h: false
      };


      // Parse the supplied command line options.
      var parsedCommandLineOptions = parseArgs( commandLineArgs, {
        boolean: true
      } );

      // Check whether there are any unrecognized options, bail if so.
      for ( var key in parsedCommandLineOptions ) {
        if ( parsedCommandLineOptions.hasOwnProperty( key ) ) {
          if ( key !== '_' && parsedCommandLineOptions.hasOwnProperty( key ) && !defaultOptions.hasOwnProperty( key ) ) {
            console.log( 'Unrecognized option: ' + key );
            console.log( 'try --help for usage information.' );
            return;
          }
        }
      }

      // Check whether help is specified.
      if ( parsedCommandLineOptions.hasOwnProperty( 'help' ) || parsedCommandLineOptions.hasOwnProperty( 'h' ) ) {
        this.printUsage();
        return;
      }

      // Make sure that withActivities is either yes, no, or both.
      if ( !!parsedCommandLineOptions.withActivities ) {
        if ( parsedCommandLineOptions.withActivities === 'no' || parsedCommandLineOptions.withActivities === 'yes' ||
             parsedCommandLineOptions.withActivities === 'both' ) {
          // These are correct, moving on
        }
        else {
          console.log( '"' + parsedCommandLineOptions.withActivities + '" is not a valid input, please enter either yes, no, or both' );
          throw new Error( 'Invalid Command Line Argument' );

        }
      }


      // Put the locales into arrays and add them to commandLineArgs.
      if ( !!parsedCommandLineOptions.locales ) {
        var locales = (parsedCommandLineOptions.locales).split( "," );

        // validate locales to make sure they are legitimate
        var localeList = require( '../../chipper/js/data/localeInfo.js' );
        var correct = false;
        // check if locale arguments are valid
        for ( var index in locales ) {
          if ( locales.hasOwnProperty( index ) ) {
            correct = false;
            for ( var currentLocale in localeList ) {
              if ( locales[ index ].toLowerCase() === currentLocale ) {
                correct = true;
              }
            }
            if ( !correct ) {
              console.log( '"' + locales[ index ] + '" is an invalid locale, please enter as many valid locales as desired, each separated by a comma' );
              throw new Error( 'Invalid Command Line Argument' );
            }
          }
        }
        parsedCommandLineOptions.locales = locales;
      }

      // Make sure that the platform arguments are either osx, windows, or linux
      // Then add them to an array and to commandLineArgs
      if ( !!parsedCommandLineOptions.platform ) {
        var platforms = (parsedCommandLineOptions.platform).split( "," );
        var defaultPlatforms = defaultOptions.platform;
        var isCorrect = false;
        // check if platform arguments are valid
        for ( var i in platforms ) {
          if ( platforms.hasOwnProperty( i ) ) {
            isCorrect = false;
            for ( var j in defaultPlatforms ) {
              if ( defaultPlatforms.hasOwnProperty( j ) ) {
                if ( platforms[ i ].toLowerCase() === defaultPlatforms[ j ] ) {
                  isCorrect = true;
                }
              }
            }
            if ( !isCorrect ) {
              console.log( '"' + platforms[ i ] + '" is an invalid platform, please enter any of the following: windows, osx, or linux separated by a comma' );
              throw new Error( 'Invalid Command Line Argument' );
            }
          }
        }
        parsedCommandLineOptions.platform = platforms;
      }

      // Merge the default and supplied options , prioritizing the parsedCommandLineOptions
      var options = _.extend( defaultOptions, parsedCommandLineOptions );

      // Kick off the build.
      this.startBuild( options );
    },

    /**
     * Print information to the console about how this should be used.  This assumes that a bash script is being used
     * to launch the build, which was true at the time of this writing.
     */
    printUsage: function() {
      console.log( 'Usage:' );
      console.log( '  phet-installer-builder.sh [options]' );
      console.log( '' );
      console.log( 'Options:' );
      console.log(
        '  --deploy (deploy the installers to the web site when built)\n' +
        '    type: bool  default: false\n' +
        '  --deployDev (deploy the installers to the development server when built)\n' +
        '    type: bool  default: false\n' +
        '  --email (deploy the installers to the development server when built)\n' +
        '    type: bool  default: false\n' +
        '  --help (print usage and exit)\n' +
        '    type: bool  default: false\n' +
        '  --withActivities (include the activities when building the installer, can be yes, no, ' +
        '                    or both, which will rip two copies of the website)\n' +
        '    type: string  default: false\n' +
        '  --locales (locales to build for, separated by comma)\n' +
        '    type: string  default: en\n' +
        '  --logFile (file name)\n' +
        '    type: string  default: undefined\n' +
        '  --platform (platforms to build for, can be any of: windows, osx, linux, separated with a comma, lowercase)\n' +
        '    type: bool  default: false\n' +
        '  --silent (do not log to console)\n' +
        '    type: bool  default: false\n'
      );
      console.log( '' );
      console.log(
        'Example - Build an installer that includes simplified Chinese and English for windows only:\n' +
        '  phet-installer-builder.sh --logFile=myLog.txt --locale=zh_CN,en --platform=windows\n' +
        '\n' +
        'Example - Build an installer that includes activities for all platforms and deploy it to the web site:\n' +
        '  phet-installer-builder.sh --logFile=./logs/log.txt --deploy --withActivities=yes --platform=all'
      );
    },

    ripAndBuild: function( options, cb ) {
      // Rip the website from phet.colorado.edu to a local temp file
      PhetWebsiteRipper.startRip( options, function() {

          // Package the ripped website into platform-specific installers.
          InstallerBuilder.packageWebsite( options );
          cb();
        }
      );
    },

    sendEmails: function() {
      // TODO email everyone that process has completed

    },

    /**
     * Initiate the build process.  This manages all the steps necessary to rip the web site, make any changes, convert
     * the rip into the multi-platform installer set, deploy the installers to the web site, and email notifications
     * about the build.
     *
     * @param {object} options - Object that defines the details of the build, see the code for details.
     * @public
     */
    startBuild: function( options ) {
      // Configure the logger for this build.

      var timestampFunction = function() { return (new Date()).toString().replace( / GMT.*\(.*\)/, "" );};
      // Add timestamps to every message
      winston.remove( winston.transports.Console );
      winston.add( winston.transports.Console, {
        'timestamp': timestampFunction
      } );

      if ( options.logFile ) {
        winston.add( winston.transports.File, { filename: options.logFile, 'timestamp': timestampFunction } );
      }
      if ( options.silent ) {
        winston.remove( winston.transports.Console );
      }
      // Log start info
      winston.log( 'info', 'Starting build.' );
      winston.log( 'info', 'build options = ' + JSON.stringify( options ) );

      // Make two completely different rips, one with activities and one without.
      // To make sure that these rips don't overwrite each other, there is an option
      // in the installerBuilder that handles renaming installer files.
      if ( options.withActivities === 'both' ) {
        winston.log( 'info', 'Starting ripping with and without activities.' );
        options.withActivities = 'no';
        this.ripAndBuild( options, function() {
          winston.log( 'info', 'Finished installer without activities' );
          options.withActivities = 'yes';
          this.ripAndBuild( options, function() {
            winston.log( 'info', 'Finished installer with activities' );
            if ( options.email ) {this.sendEmails();}
          } );
        } );
      }
      // If withActivities isn't both, then just do the one, normal rip.
      else {
        this.ripAndBuild( options, function() {
          if ( options.email ) {this.sendEmails();}
        } );
      }
    }

  };
} );