// Copyright 2002-2015, University of Colorado Boulder

/**
 * An object that takes a static copy of a the PhET website and packages it into installers for Windows, OSX, and
 * Linux, depending on the platform option.
 *
 * This object is highly dependent upon the BitRock installbuilder software.  If a different install builder utility is
 * used, this file will need to change substantially.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var fs = require( 'fs' );
  var InstallerBuilderSharedConstants = require( 'InstallerBuilderSharedConstants' );
  var BuilderUtils = require( 'BuilderUtils' );
  var XMLUtils = require( 'XMLUtils' );
  var winston = require( 'winston' );
  var ShellJS = require( 'shelljs' );
  var xml2js = require( 'xml2js' );

  // constants
  var INSTALL_BUILDER = InstallerBuilderSharedConstants.INSTALL_BUILDER;
  var TEMPLATE_BITROCK_XML = fs.readFileSync( 'data/bitrock-project-template.xml' );
  var PROJECT_FILE_PATH = InstallerBuilderSharedConstants.TEMP_DIR + 'bitrock-project-file.xml';
  var RIP_BASE_DIR = InstallerBuilderSharedConstants.RIP_DIR + InstallerBuilderSharedConstants.HOST_NAME + '/';
  var TARGET_PLATFORMS = [ 'windows', 'osx', 'linux' ];

  // Only needed by the tiny installer, because buildInstaller() loads these dynamically
  var DISTRIBUTION_DIRS_FOR_TINY_INSTALLER = []; // No distribution directories for this rip.
  var ADDITIONAL_DISTRIBUTION_FILES_FOR_TINY_INSTALLER = [ 'index.html' ]; // TODO: Should version.html be in here too?


  // static object, no constructor
  return {

    /**
     * Build a 'tiny' version of the installer, meaning one that only include the home page of the website.
     * This is used to page the rip made by PhetWebsiteRipper.ripHomePageOnly()
     * @public
     */
    buildTinyInstallers: function() {

      winston.log( 'info', 'Building the tiny installers.' );
      winston.log( 'info', 'Writing BitRock project file to ' + PROJECT_FILE_PATH );

      // Convert the XML-formatted project description into a JavaScript object so that it can be manipulated.
      var projectDescriptionJS;
      // TODO: I believe that the callback in the following is executed synchronously, but need to verify if kept.
      xml2js.parseString( TEMPLATE_BITROCK_XML, function( err, result ) {
        if ( !err ) {
          projectDescriptionJS = result;
        }
        else {
          winston.log( "error", "Couldn't parse TEMPLATE_BITROCK_XML, nullPointer imminent:   " + err );
        }
      } );

      // Set the output directory.
      projectDescriptionJS.project.outputDirectory = InstallerBuilderSharedConstants.INSTALLER_OUTPUT_DIR;

      // Add the distribution directories, i.e. the directories captured in the rip that need to be packaged into the
      // installers, to the project description.
      var distributionDirectories = [];
      DISTRIBUTION_DIRS_FOR_TINY_INSTALLER.forEach( function( dirName ) {
        // If the format for the object strikes you as odd, it's because of the requirements of the JavaScript-to-XML
        // conversion process.
        distributionDirectories.push( { origin: [ RIP_BASE_DIR + dirName ] } );
      } );
      var distributionFileList = projectDescriptionJS.project.componentList[ 0 ].component[ 0 ].folderList[ 0 ].folder[ 0 ].distributionFileList;
      if ( distributionDirectories.length > 0 ) {
        distributionFileList[ 0 ].distributionDirectory = distributionDirectories;
      }

      // Add the additional distribution files, i.e. the files captured in the rip that need to be packaged into the
      // installers, to the project description.
      ADDITIONAL_DISTRIBUTION_FILES_FOR_TINY_INSTALLER.forEach( function( fileName ) {
        distributionFileList[ 0 ].distributionFile.push( { origin: [ RIP_BASE_DIR + fileName ] } );
      } );

      // Output the BitRock install builder project description as an XML file.
      var xmlBuilder = new xml2js.Builder();
      var projectDescriptionXML = xmlBuilder.buildObject( projectDescriptionJS );
      fs.writeFileSync( PROJECT_FILE_PATH, projectDescriptionXML );

      // Assemble the command for building the installers and execute it for each of the target platforms.
      var coreBuildCommand = INSTALL_BUILDER + ' build ' + ' \"' + PROJECT_FILE_PATH + '\"';
      TARGET_PLATFORMS.forEach( function( targetPlatform ) {
        var buildCommand = coreBuildCommand + ' ' + targetPlatform;
        winston.log( 'info', 'Initiating build of installer for ' + targetPlatform + ', command = ' + buildCommand );
        var buildResult = ShellJS.exec( buildCommand );
        if ( buildResult.code !== 0 ) {
          winston.log( 'error', 'Build of installer failed, command result = ' + buildResult.output );
          throw new Error( 'Aborting build due to failure of BitRock installer build step.' );
        }
        winston.log( 'info', 'Build succeeded for ' + targetPlatform + ', buildResult = ' + buildResult.output );
      } );
      winston.log( 'info', 'Build succeeded for all targeted platforms' );
    },


    /**
     * Build the installer.
     * Handles which platform to make the installer into,
     * and puts output into a folder specific to quick, with/o activities.
     * @param options: given by the build manager
     * @public
     */
    buildInstaller: function( options ) {

      winston.log( 'info', 'Building installer.' );
      winston.log( 'info', 'Writing BitRock project file to ' + PROJECT_FILE_PATH );

      // Convert the XML-formatted project description into a JavaScript object so that it can be manipulated.
      var projectDescriptionJS = XMLUtils.parseXMLFile( PROJECT_FILE_PATH );

      // Set the output Directory for the installer
      projectDescriptionJS.project.outputDirectory = InstallerBuilderSharedConstants.INSTALLER_OUTPUT_DIR;
      var filename = projectDescriptionJS.project.installerFilename.toString();
      // Set the output filename, making it unique depending on activities and if it's a quick build.
      // If there are no activities, then the file will be named with no suffix
      if ( options.quick ) {
        projectDescriptionJS.project.installerFilename = filename.replace( '${platform_name}', '${platform_name}-quick' );
      }
      else if ( options.withActivities === 'yes' ) {
        projectDescriptionJS.project.installerFilename = filename.replace( '${platform_name}', '${platform_name}-activities' );
      }


      // Dynamically loads the files and directories from the rip
      var filesAndDirectories = BuilderUtils.findDistributionsFilesAndDirectories();

      var distributionFileList = projectDescriptionJS.project.componentList[ 0 ].component[ 0 ].folderList[ 0 ].folder[ 0 ].distributionFileList;

      /*
       * Add the distribution directories, i.e. the directories captured in the rip that need to be packaged into the
       * installers, to the project description.
       */
      var distributionDirectories = [];
      filesAndDirectories.directories.forEach( function( dirName ) {
        //If the format for the object strikes you as odd, it's because of the requirements of the JavaScript-to-XML
        // conversion process.
        distributionDirectories.push( { origin: [ RIP_BASE_DIR + dirName ] } );
      } );

      if ( distributionDirectories.length > 0 ) {
        distributionFileList[ 0 ].distributionDirectory = distributionDirectories;
      }

      // Add the additional distribution files, i.e. the files captured in the rip that need to be packaged into the
      // installers, to the project description.
      filesAndDirectories.files.forEach( function( fileName ) {
        distributionFileList[ 0 ].distributionFile.push( { origin: [ RIP_BASE_DIR + fileName ] } );
      } );


      // Once the description is loaded with everything,
      // and output the BitRock install builder project description as an XML file.
      XMLUtils.buildAndWriteXML( projectDescriptionJS, PROJECT_FILE_PATH );

      // Assemble the command for building the installers and execute it for each of the target platforms.
      var coreBuildCommand = INSTALL_BUILDER + ' build ' + ' \"' + PROJECT_FILE_PATH + '\"';
      options.platform.forEach( function( targetPlatform ) {
        BuilderUtils.buildInstallerForSelectPlatform( targetPlatform, coreBuildCommand );
      } );
      winston.log( 'info', 'Build succeeded for all targeted platforms' );
    },

    tempTest: function() {
      BuilderUtils.findDistributionsFilesAndDirectories();
    },

    /**
     * facilitates the BitRock installBuilder side of the project
     * Includes some options handling and version logging.
     * @param options: passed in from Build Manager.
     */
    packageWebsite: function( options ) {

      BuilderUtils.logInstallerBuilderVersion();

      // Put the template into the temp path, to be loaded and changed, then used in the installer
      fs.writeFileSync( PROJECT_FILE_PATH, TEMPLATE_BITROCK_XML );

      this.buildInstaller( options );
    }

  };


} );