// Copyright 2002-2014, University of Colorado Boulder

/**
 * An object that creates a static snapshot of the PhET website, also known as doing a 'rip'.  There are multiple
 * different types of rips that are supported, from very quick and small ones (generally used for testing) to more
 * complete ones.
 *
 * This object is highly dependent upon the ripper utility HTTrack.
 *
 * @author John Blanco
 */
define( function( require ) {
  'use strict';

  // modules
  var InstallerBuilderSharedConstants = require( 'InstallerBuilderSharedConstants' );
  var RipperUtils = require( 'RipperUtils' );
  var ShellJS = require( 'shelljs' );
  var winston = require( 'winston' );


  // constants
  var RIPPER = InstallerBuilderSharedConstants.RIPPER;
  var WEB_ROOT = InstallerBuilderSharedConstants.WEB_ROOT;
  var RIP_DIR = InstallerBuilderSharedConstants.RIP_DIR;
  var SPONSOR_LIST_URL = InstallerBuilderSharedConstants.SPONSOR_LIST_URL;
  var PHET_HOSTNAME = InstallerBuilderSharedConstants.HOST_NAME;
  var PHET_DOWNLOADS_HOSTNAME = InstallerBuilderSharedConstants.DOWNLOADS_HOST_NAME;

  /*
   Special Settings that probably want to be used for all rips, not totally sure though.
   F user-agent-name
   %eN set the external links depth to N
   rN set the mirror depth to N
   sN follow robots.txt and meta robots tags (0=never,1=sometimes,* 2=always) (--robots[=N])
   AN maximum transfer rate in bytes/seconds
   no documentation found on --disable-security-limits
   */
  var DEFAULT_RIPPER_OPTIONS = ' -%e0 -r10 -s0 -A10000000000 --disable-security-limits ';


  /**
   * Helper functions for the ripper module
   * @private
   *
   */
  var createNewFilterItem = function( controlChar, hostName, path ) {
    return ' "' + controlChar + hostName + "/" + path + '" ';
  };
  var createNewLocaleSpecificFilterItem = function( controlChar, hostName, path, locales ) {
    var returnFilter = " ";
    for ( var i in locales ) {
      if ( locales.hasOwnProperty( i ) ) {
        returnFilter += '"' + controlChar + hostName + "/" + locales[ i ] + '/' + path + '" ';
      }
    }
    return returnFilter;
  };

  return {

    /**
     * Perform a rip of the site including activities.
     * @public
     */
    ripWithActivities: function( options, cb ) {
      winston.log( 'info', 'Ripping Website With Activities' );

      var ripper_user_agent = RipperUtils.setUserAgent( false, options.locales );

      var ripperConfig = createNewFilterItem( '+', PHET_DOWNLOADS_HOSTNAME, '*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'files/video/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'workshops/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'installer/*' ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'get-phet/*', options.locales ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'subscribe/*', options.locales ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'teaching-resources/submit-activity', options.locales ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'teaching-resources/manage-activities', options.locales ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.audio!*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.image!*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.mipmap!*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.jar' );


      var ripCommand = RIPPER + ' "' + WEB_ROOT + '" "' + SPONSOR_LIST_URL + '" -O ' + RIP_DIR + " " +
                       ripperConfig + ripper_user_agent + DEFAULT_RIPPER_OPTIONS;

      RipperUtils.ripWebsite( ripCommand, function() {
        RipperUtils.removeExtraHTMLFiles();
        RipperUtils.fixAllHTML5Sims( function() {
          RipperUtils.fixAllJavaSims( cb );
        } );
      } );

    },


    /**
     * Perform a rip of the site not including activities.
     * @public
     */
    ripWithoutActivities: function( options, cb ) {
      winston.log( 'info', 'Ripping Website Without Activities' );

      var ripper_user_agent = RipperUtils.setUserAgent( true, options.locales );

      var ripperConfig = createNewFilterItem( '-', PHET_HOSTNAME, 'files/activities/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'files/video/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'workshops/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'installer/*' ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'get-phet/*', options.locales ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'subscribe/*', options.locales ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'teaching-resources/submit-activity', options.locales ) +
                         createNewLocaleSpecificFilterItem( '-', PHET_HOSTNAME, 'teaching-resources/manage-activities', options.locales ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*download-servlet' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.jar' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.audio!*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.image!*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '*.mipmap!*' );


      var ripCommand = RIPPER + ' "' + WEB_ROOT + '" "' + SPONSOR_LIST_URL + '" -O ' + RIP_DIR + " " +
                       ripperConfig + ripper_user_agent + DEFAULT_RIPPER_OPTIONS;

      RipperUtils.ripWebsite( ripCommand, function() {
        RipperUtils.removeExtraHTMLFiles();
        RipperUtils.fixAllHTML5Sims( function() {
          RipperUtils.fixAllJavaSims( cb );
        } );
      } );

    },

    /**
     * Perform a small rip that captures the home page and a single HTML5, Java, and Flash sim.  This is used
     * primarily for fast-turnaround testing.
     * @public
     */
    ripOneOfEachSimType: function( cb ) {
      winston.log( 'info', 'Ripping Quick Version Of Website' );

      var HTML_SIM = 'molecules-and-light';
      var JAVA_SIM = 'build-an-atom';
      var FLASH_SIM = 'pendulum-lab';


      // A basic website to be able to get to the sims, but not much more
      var ripperConfig = createNewFilterItem( '-', PHET_HOSTNAME, 'files/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, '2013/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'blog/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'images/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'resources/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'workshops/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'troubleshooting/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'installer/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'about/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'newsletters/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'publications/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/get-phet/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/about/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/for-translators/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/get-subscribe/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/for-teachers/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/teaching-resources/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/contributions/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/troubleshooting/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/simulation/legacy/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/simulation/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'en/simulations/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'sims/*' ) +
                         createNewFilterItem( '-', PHET_HOSTNAME, 'sims/html/*' ) +
                         createNewFilterItem( '+', PHET_HOSTNAME, 'en/simulations/category/new*' ) +

                         // For Java Sim
                         createNewFilterItem( '+', PHET_HOSTNAME, 'sims/' + JAVA_SIM + '/*' ) +
                         createNewFilterItem( '+', PHET_HOSTNAME, 'en/simulation/*' + JAVA_SIM + '*' ) +

                         // For HTML Simx
                         createNewFilterItem( '+', PHET_HOSTNAME, 'sims/html/' + HTML_SIM + '/*' ) +
                         createNewFilterItem( '-', '', '*.audio!*' ) +
                         createNewFilterItem( '-', '', '*.image!*' ) +
                         createNewFilterItem( '-', '', '*.mipmap!*' ) +
                         createNewFilterItem( '+', PHET_HOSTNAME, 'en/simulation/*' + HTML_SIM + '*' ) +

                         // For Flash Sim
                         createNewFilterItem( '+', PHET_HOSTNAME, 'sims/' + FLASH_SIM + '/' + FLASH_SIM + '_en.html' ) +
                         createNewFilterItem( '+', PHET_HOSTNAME, 'en/simulation/*' + FLASH_SIM + '*' );

      //add user-agent
      ripperConfig += RipperUtils.setUserAgent( false, [] );

      var ripCommand = RIPPER + ' "' + WEB_ROOT + '" "' + SPONSOR_LIST_URL + '" -O ' + RIP_DIR + ' ' + ripperConfig + DEFAULT_RIPPER_OPTIONS;

      RipperUtils.ripWebsite( ripCommand, function() {
        RipperUtils.ripOneHTML5SimEn( HTML_SIM, function() {
          RipperUtils.ripOneJavaSimEn( JAVA_SIM, cb );
        } );
      } );
    },


    /**
     * Perform a very small rip that captures only the PhET home page.  This is used primarily for fast-turnaround testing.
     * @public
     */
    ripHomePageOnly: function() {

      // Remove any previous rip that might be lying around.  TODO: Test if updates can be made to work - might save time.
      RipperUtils.removePreviousRip();

      // Assemble the ripper command and execute it.
      //-r1 means only one level deep in the mirror rip, meaning only the WEB_ROOT
      var ripCommand = RIPPER + ' ' + WEB_ROOT + ' -O ' + RIP_DIR + ' -r1';
      winston.log( 'info', 'About to start rip, ripCommand = ' + ripCommand );
      var ripResult = ShellJS.exec( ripCommand );
      if ( ripResult.code !== 0 ) {
        // Rip command failed.  Note the error and abort.
        winston.log( 'error', 'Rip command failed, command output = ' + ripResult.output );
        throw new Error( 'Build aborted due to failure of rip.' );
      }
      winston.log( 'info', 'Rip completed, rip command output = ' + ripResult.output );
    },


    /**
     * This method handles the ripping of the website. Removing any old rips, and dealing with options.
     * @param options - for easily making custom builds
     * @param cb
     */
    startRip: function( options, cb ) {
      RipperUtils.removePreviousRip();
      winston.log( 'info', 'RIPPER: ' + RIPPER + '  WEB_ROOT: ' + WEB_ROOT );


      if ( options.quick ) {
        this.ripOneOfEachSimType( cb );
      }
      else if ( options.withActivities === 'yes' ) {
        this.ripWithActivities( options, cb );
      }
      else {
        this.ripWithoutActivities( options, cb );
      }
    }
  };
} );